/*
* Copyright 2009, KyTea Development Team
* 
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
* 
*     http://www.apache.org/licenses/LICENSE-2.0
* 
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#ifndef CORPUS_IO_H__ 
#define CORPUS_IO_H__ 

#include "general-io.h"
#include "kytea-struct.h"

using namespace std;

namespace kytea {

class CorpusIO : public GeneralIO {

public:

    typedef char Format;
    const static Format FORMAT_RAW  = 0;
    const static Format FORMAT_FULL = 1;
    const static Format FORMAT_PART = 2;
    const static Format FORMAT_PROB = 3;

    CorpusIO(StringUtil * util) : GeneralIO(util) { }
    CorpusIO(StringUtil * util, const char* file, bool out) : GeneralIO(util,file,out,false) { } 
    CorpusIO(StringUtil * util, iostream & str, bool out) : GeneralIO(util,str,out,false) { }

    virtual ~CorpusIO() { }

    // create an appropriate parser based on the type
    static CorpusIO* createIO(const char* file, Format form, bool output, StringUtil* util);
    static CorpusIO* createIO(iostream & str, Format form, bool output, StringUtil* util);

    virtual KyteaSentence * readSentence() = 0;
    virtual void writeSentence(const KyteaSentence * sent, double conf = 0.0) = 0;

};

class FullCorpusIO : public CorpusIO {

private:

    KyteaString bounds_;
#define FULL_BOUND " /"

public:
    FullCorpusIO(StringUtil * util) : CorpusIO(util) { 
        bounds_ = util_->mapString(FULL_BOUND); 
    }
    FullCorpusIO(const CorpusIO & c) : CorpusIO(c) { 
        bounds_ = util_->mapString(FULL_BOUND); 
    }
    FullCorpusIO(StringUtil * util, const char* file, bool out) : CorpusIO(util,file,out) { 
        bounds_ = util_->mapString(FULL_BOUND); 
    } 
    FullCorpusIO(StringUtil * util, iostream & str, bool out) : CorpusIO(util,str,out) { 
        bounds_ = util_->mapString(FULL_BOUND);
    }
    
    KyteaSentence * readSentence();
    void writeSentence(const KyteaSentence * sent, double conf = 0.0);

};

class PartCorpusIO : public CorpusIO {
    
private:

#define PART_BOUND " -|/"
    KyteaString bounds_;

public:
    PartCorpusIO(StringUtil * util) : CorpusIO(util) { 
        bounds_ = util_->mapString(PART_BOUND);
    }
    PartCorpusIO(const CorpusIO & c) : CorpusIO(c) { 
        bounds_ = util_->mapString(PART_BOUND);
    }
    PartCorpusIO(StringUtil * util, const char* file, bool out) : CorpusIO(util,file,out) { 
        bounds_ = util_->mapString(PART_BOUND);
    }
    PartCorpusIO(StringUtil * util, iostream & str, bool out) : CorpusIO(util,str,out) { 
        bounds_ = util_->mapString(PART_BOUND);
    }
    
    KyteaSentence * readSentence();
    void writeSentence(const KyteaSentence * sent, double conf = 0.0);

};

class ProbCorpusIO : public FullCorpusIO {

public:
    ProbCorpusIO(StringUtil * util) : FullCorpusIO(util) { }
    ProbCorpusIO(const CorpusIO & c) : FullCorpusIO(c) { }
    ProbCorpusIO(StringUtil * util, const char* file, bool out) : FullCorpusIO(util,file,out) { } 
    ProbCorpusIO(StringUtil * util, iostream & str, bool out) : FullCorpusIO(util,str,out) { }

    KyteaSentence * readSentence();
    void writeSentence(const KyteaSentence * sent, double conf = 0.0);

};

class RawCorpusIO : public CorpusIO {

public:
    RawCorpusIO(StringUtil * util) : CorpusIO(util) { }
    RawCorpusIO(const CorpusIO & c) : CorpusIO(c) { }
    RawCorpusIO(StringUtil * util, const char* file, bool out) : CorpusIO(util,file,out) { } 
    RawCorpusIO(StringUtil * util, iostream & str, bool out) : CorpusIO(util,str,out) { }

    KyteaSentence * readSentence();
    void writeSentence(const KyteaSentence * sent, double conf = 0.0);

};


}

#endif
